@@ -101,6 +101,8 @@
 #include <gtk/gtkfilefilter.h>
 #include <gtk/gtkfontbutton.h>
 #include <gtk/gtkfontsel.h>
+#include <gtk/gtkfontchooser.h>
+#include <gtk/gtkfontchooserdialog.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkgradient.h>
 #include <gtk/gtkgrid.h>
@@ -141,6 +143,7 @@
 #include <gtk/gtknumerableicon.h>
 #include <gtk/gtkoffscreenwindow.h>
 #include <gtk/gtkorientable.h>
+#include <gtk/gtkoverlay.h>
 #include <gtk/gtkpagesetup.h>
 #include <gtk/gtkpapersize.h>
 #include <gtk/gtkpaned.h>
