@@ -25,7 +25,6 @@
 #ifndef __GTK_BUILDER_H__
 #define __GTK_BUILDER_H__
 
-#include <gdkconfig.h>
 #include <glib-object.h>
 
 G_BEGIN_DECLS
@@ -85,7 +84,7 @@ struct _GtkBuilder
 {
   GObject parent_instance;
 
-  GtkBuilderPrivate *GSEAL (priv);
+  GtkBuilderPrivate *priv;
 };
 
 struct _GtkBuilderClass
