@@ -31,7 +31,7 @@
 
 #include "gtkbutton.h"
 #include "gtkdialog.h"
-#include "gtkhbbox.h"
+#include "gtkbbox.h"
 #include "gtklabel.h"
 #include "gtkmarshalers.h"
 #include "gtkvbox.h"
@@ -56,7 +56,7 @@
  * be packed. The bottom area is known as the
  * <structfield>action_area</structfield>. This is generally used for
  * packing buttons into the dialog which may perform functions such as
- * cancel, ok, or apply. The two areas are separated by a #GtkHSeparator.
+ * cancel, ok, or apply.
  *
  * #GtkDialog boxes are created with a call to gtk_dialog_new() or
  * gtk_dialog_new_with_buttons(). gtk_dialog_new_with_buttons() is
@@ -241,6 +241,8 @@ gtk_dialog_class_init (GtkDialogClass *class)
   widget_class->map = gtk_dialog_map;
   widget_class->style_updated = gtk_dialog_style_updated;
 
+  gtk_widget_class_set_accessible_role (widget_class, ATK_ROLE_DIALOG);
+
   class->close = gtk_dialog_close;
 
   g_type_class_add_private (gobject_class, sizeof (GtkDialogPrivate));
@@ -349,6 +351,7 @@ update_spacings (GtkDialog *dialog)
                         "action-area-border", &action_area_border,
                         NULL);
 
+  
   gtk_container_set_border_width (GTK_CONTAINER (priv->vbox),
                                   content_area_border);
   if (!_gtk_box_get_spacing_set (GTK_BOX (priv->vbox)))
@@ -397,6 +400,8 @@ gtk_dialog_init (GtkDialog *dialog)
   gtk_window_set_type_hint (GTK_WINDOW (dialog),
 			    GDK_WINDOW_TYPE_HINT_DIALOG);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+
+  update_spacings (dialog);
 }
 
 static GtkBuildableIface *parent_buildable_iface;
@@ -508,6 +513,8 @@ gtk_dialog_map (GtkWidget *widget)
 static void
 gtk_dialog_style_updated (GtkWidget *widget)
 {
+  GTK_WIDGET_CLASS (gtk_dialog_parent_class)->style_updated (widget);
+
   update_spacings (GTK_DIALOG (widget));
 }
 
