@@ -26,8 +26,8 @@
 #include "gtksettings.h"
 #include "gtkdebug.h"
 #include "gtkprivate.h" /* GTK_LIBDIR */
-#include "gtkintl.h" 
-#include "gtkalias.h"
+#include "gtkmainprivate.h"
+#include "gtkintl.h"
 
 #include <gmodule.h>
 
@@ -237,7 +237,16 @@ find_module (const gchar *name)
     }
 
   module = g_module_open (module_name, G_MODULE_BIND_LOCAL | G_MODULE_BIND_LAZY);
-  g_free(module_name);
+
+  if (_gtk_module_has_mixed_deps (module))
+    {
+      g_warning ("GTK+ module %s cannot be loaded.\n"
+                 "GTK+ 2.x symbols detected. Using GTK+ 2.x and GTK+ 3 in the same process is not supported.", module_name);
+      g_module_close (module);
+      module = NULL;
+    }
+
+  g_free (module_name);
 
   return module;
 }
@@ -249,6 +258,18 @@ cmp_module (GtkModuleInfo *info,
   return info->module != module;
 }
 
+static gboolean
+module_is_blacklisted (const gchar *name)
+{
+  if (g_str_equal (name, "gail"))
+    {
+      g_message ("Not loading module \"gail\": The functionality is provided by GTK natively. Please try to not load it.");
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static GSList *
 load_module (GSList      *module_list,
 	     const gchar *name)
@@ -265,7 +286,7 @@ load_module (GSList      *module_list,
       for (l = gtk_modules; l; l = l->next)
 	{
 	  info = l->data;
-	  if (g_slist_find_custom (info->names, name, 
+	  if (g_slist_find_custom (info->names, name,
 				   (GCompareFunc)strcmp))
 	    {
 	      info->ref_count++;
@@ -274,13 +295,20 @@ load_module (GSList      *module_list,
 	    }
 	}
 
-      if (!success) 
+      if (!success)
 	{
 	  module = find_module (name);
 
 	  if (module)
 	    {
-	      if (g_module_symbol (module, "gtk_module_init", &modinit_func_ptr))
+              /* Do the check this late so we only warn about existing modules,
+               * not old modules that are still in the modules path. */
+              if (module_is_blacklisted (name))
+                {
+                  modinit_func = NULL;
+                  success = TRUE;
+                }
+              else if (g_module_symbol (module, "gtk_module_init", &modinit_func_ptr))
 		modinit_func = modinit_func_ptr;
 	      else
 		modinit_func = NULL;
@@ -313,7 +341,7 @@ load_module (GSList      *module_list,
 		      gtk_modules = g_slist_append (gtk_modules, info);
 		      
 		      /* display_init == NULL indicates a non-multihead aware module.
-		       * For these, we delay the call to init_func until first display is 
+		       * For these, we delay the call to init_func until first display is
 		       * opened, see default_display_notify_cb().
 		       * For multihead aware modules, we call init_func immediately,
 		       * and also call display_init_func on all opened displays.
@@ -354,8 +382,13 @@ load_module (GSList      *module_list,
 	}
     }
   else
-    g_message ("Failed to load module \"%s\": %s", name, g_module_error ());
-  
+    {
+      const gchar *error = g_module_error ();
+
+      g_message ("Failed to load module \"%s\"%s%s",
+                 name, error ? ": " : "", error ? error : "");
+    }
+
   return module_list;
 }
 
