@@ -1679,9 +1679,15 @@ gtk_status_icon_padding_changed (GtkStatusIcon *status_icon)
   padding = _gtk_tray_icon_get_padding (GTK_TRAY_ICON (priv->tray_icon));
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    gtk_misc_set_padding (GTK_MISC (priv->image), padding, 0);
+    {
+      gtk_widget_set_margin_left (priv->image, padding);
+      gtk_widget_set_margin_right (priv->image, padding);
+    }
   else
-    gtk_misc_set_padding (GTK_MISC (priv->image), 0, padding);
+    {
+      gtk_widget_set_margin_bottom (priv->image, padding);
+      gtk_widget_set_margin_top (priv->image, padding);
+    }
 }
 
 static void
@@ -2129,7 +2135,7 @@ gtk_status_icon_get_pixbuf (GtkStatusIcon *status_icon)
  *
  * Since: 2.10
  **/
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_stock (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2161,7 +2167,7 @@ gtk_status_icon_get_stock (GtkStatusIcon *status_icon)
  *
  * Since: 2.10
  **/
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_icon_name (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2914,7 +2920,7 @@ gtk_status_icon_set_title (GtkStatusIcon *status_icon,
  *
  * Since: 2.18
  */
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_title (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
