@@ -52,7 +52,8 @@ enum {
   PROP_ERROR_COLOR,
   PROP_WARNING_COLOR,
   PROP_SUCCESS_COLOR,
-  PROP_PADDING
+  PROP_PADDING,
+  PROP_ICON_SIZE
 };
 
 struct _GtkTrayIconPrivate
@@ -66,16 +67,18 @@ struct _GtkTrayIconPrivate
   Atom visual_atom;
   Atom colors_atom;
   Atom padding_atom;
+  Atom icon_size_atom;
   Window manager_window;
   GdkVisual *manager_visual;
   gboolean manager_visual_rgba;
 
   GtkOrientation orientation;
-  GdkColor fg_color;
-  GdkColor error_color;
-  GdkColor warning_color;
-  GdkColor success_color;
+  GdkRGBA fg_color;
+  GdkRGBA error_color;
+  GdkRGBA warning_color;
+  GdkRGBA success_color;
   gint padding;
+  gint icon_size;
 };
 
 static void gtk_tray_icon_constructed   (GObject     *object);
@@ -133,7 +136,7 @@ gtk_tray_icon_class_init (GtkTrayIconClass *class)
                                    g_param_spec_boxed ("fg-color",
                                                        P_("Foreground color"),
                                                        P_("Foreground color for symbolic icons"),
-                                                       GDK_TYPE_COLOR,
+                                                       GDK_TYPE_RGBA,
                                                        GTK_PARAM_READABLE));
 
   g_object_class_install_property (gobject_class,
@@ -141,7 +144,7 @@ gtk_tray_icon_class_init (GtkTrayIconClass *class)
                                    g_param_spec_boxed ("error-color",
                                                        P_("Error color"),
                                                        P_("Error color for symbolic icons"),
-                                                       GDK_TYPE_COLOR,
+                                                       GDK_TYPE_RGBA,
                                                        GTK_PARAM_READABLE));
 
   g_object_class_install_property (gobject_class,
@@ -149,7 +152,7 @@ gtk_tray_icon_class_init (GtkTrayIconClass *class)
                                    g_param_spec_boxed ("warning-color",
                                                        P_("Warning color"),
                                                        P_("Warning color for symbolic icons"),
-                                                       GDK_TYPE_COLOR,
+                                                       GDK_TYPE_RGBA,
                                                        GTK_PARAM_READABLE));
 
   g_object_class_install_property (gobject_class,
@@ -157,7 +160,7 @@ gtk_tray_icon_class_init (GtkTrayIconClass *class)
                                    g_param_spec_boxed ("success-color",
                                                        P_("Success color"),
                                                        P_("Success color for symbolic icons"),
-                                                       GDK_TYPE_COLOR,
+                                                       GDK_TYPE_RGBA,
                                                        GTK_PARAM_READABLE));
 
   g_object_class_install_property (gobject_class,
@@ -170,6 +173,16 @@ gtk_tray_icon_class_init (GtkTrayIconClass *class)
                                                      0,
 						     GTK_PARAM_READABLE));
 
+  g_object_class_install_property (gobject_class,
+				   PROP_ICON_SIZE,
+				   g_param_spec_int ("icon-size",
+						     P_("Icon Size"),
+						     P_("The pixel size that icons should be forced to, or zero"),
+						     0,
+                                                     G_MAXINT,
+                                                     0,
+						     GTK_PARAM_READABLE));
+
   g_type_class_add_private (class, sizeof (GtkTrayIconPrivate));
 }
 
@@ -181,19 +194,24 @@ gtk_tray_icon_init (GtkTrayIcon *icon)
 
   icon->priv->stamp = 1;
   icon->priv->orientation = GTK_ORIENTATION_HORIZONTAL;
-  icon->priv->fg_color.red        = 0x0000;
-  icon->priv->fg_color.green      = 0x0000;
-  icon->priv->fg_color.blue       = 0x0000;
-  icon->priv->error_color.red     = 0xcc00;
-  icon->priv->error_color.green   = 0x0000;
-  icon->priv->error_color.blue    = 0x0000;
-  icon->priv->warning_color.red   = 0xf500;
-  icon->priv->warning_color.green = 0x7900;
-  icon->priv->warning_color.blue  = 0x3e00;
-  icon->priv->success_color.red   = 0x4e00;
-  icon->priv->success_color.green = 0x9a00;
-  icon->priv->success_color.blue  = 0x0600;
+  icon->priv->fg_color.red        = 0.0;
+  icon->priv->fg_color.green      = 0.0;
+  icon->priv->fg_color.blue       = 0.0;
+  icon->priv->fg_color.alpha      = 1.0;
+  icon->priv->error_color.red     = 0.7968;
+  icon->priv->error_color.green   = 0.0;
+  icon->priv->error_color.blue    = 0.0;
+  icon->priv->error_color.alpha   = 1.0;
+  icon->priv->warning_color.red   = 0.9570;
+  icon->priv->warning_color.green = 0.4726;
+  icon->priv->warning_color.blue  = 0.2421;
+  icon->priv->warning_color.alpha = 1.0;
+  icon->priv->success_color.red   = 0.3047;
+  icon->priv->success_color.green = 0.6016;
+  icon->priv->success_color.blue  = 0.0234;
+  icon->priv->success_color.alpha = 1.0;
   icon->priv->padding = 0;
+  icon->priv->icon_size = 0;
 
   gtk_widget_set_app_paintable (GTK_WIDGET (icon), TRUE);
   gtk_widget_add_events (GTK_WIDGET (icon), GDK_PROPERTY_CHANGE_MASK);
@@ -239,6 +257,10 @@ gtk_tray_icon_constructed (GObject *object)
 					 "_NET_SYSTEM_TRAY_PADDING",
 					 False);
 
+  icon->priv->icon_size_atom = XInternAtom (xdisplay,
+                                            "_NET_SYSTEM_TRAY_ICON_SIZE",
+                                            False);
+
   /* Add a root window filter so that we get changes on MANAGER */
   gdk_window_add_filter (root_window,
 			 gtk_tray_icon_manager_filter, icon);
@@ -275,6 +297,8 @@ gtk_tray_icon_dispose (GObject *object)
   gtk_tray_icon_clear_manager_window (icon);
 
   gdk_window_remove_filter (root_window, gtk_tray_icon_manager_filter, icon);
+
+  G_OBJECT_CLASS (gtk_tray_icon_parent_class)->dispose (object);
 }
 
 static void
@@ -305,6 +329,9 @@ gtk_tray_icon_get_property (GObject    *object,
     case PROP_PADDING:
       g_value_set_int (value, icon->priv->padding);
       break;
+    case PROP_ICON_SIZE:
+      g_value_set_int (value, icon->priv->icon_size);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -360,7 +387,7 @@ gtk_tray_icon_draw (GtkWidget *widget,
     retval = GTK_WIDGET_CLASS (gtk_tray_icon_parent_class)->draw (widget, cr);
 
   focus_child = gtk_container_get_focus_child (GTK_CONTAINER (widget));
-  if (focus_child && gtk_widget_has_focus (focus_child))
+  if (focus_child && gtk_widget_has_visible_focus (focus_child))
     {
       GtkStyleContext *context;
       GtkStateFlags state;
@@ -529,48 +556,48 @@ gtk_tray_icon_get_colors_property (GtkTrayIcon *icon)
 
   if (type == XA_CARDINAL && nitems == 12 && format == 32)
     {
-      GdkColor color;
+      GdkRGBA color;
 
       g_object_freeze_notify (G_OBJECT (icon));
 
-      color.red = prop.prop[0];
-      color.green = prop.prop[1];
-      color.blue = prop.prop[2];
+      color.red = prop.prop[0] / 65535.0;
+      color.green = prop.prop[1] / 65535.0;
+      color.blue = prop.prop[2] / 65535.0;
 
-      if (!gdk_color_equal (&icon->priv->fg_color, &color))
+      if (!gdk_rgba_equal (&icon->priv->fg_color, &color))
         {
           icon->priv->fg_color = color;
 
           g_object_notify (G_OBJECT (icon), "fg-color");
         }
 
-      color.red = prop.prop[3];
-      color.green = prop.prop[4];
-      color.blue = prop.prop[5];
+      color.red = prop.prop[3] / 65535.0;
+      color.green = prop.prop[4] / 65535.0;
+      color.blue = prop.prop[5] / 65535.0;
 
-      if (!gdk_color_equal (&icon->priv->error_color, &color))
+      if (!gdk_rgba_equal (&icon->priv->error_color, &color))
         {
           icon->priv->error_color = color;
 
           g_object_notify (G_OBJECT (icon), "error-color");
         }
 
-      color.red = prop.prop[6];
-      color.green = prop.prop[7];
-      color.blue = prop.prop[8];
+      color.red = prop.prop[6] / 65535.0;
+      color.green = prop.prop[7] / 65535.0;
+      color.blue = prop.prop[8] / 65535.0;
 
-      if (!gdk_color_equal (&icon->priv->warning_color, &color))
+      if (!gdk_rgba_equal (&icon->priv->warning_color, &color))
         {
           icon->priv->warning_color = color;
 
           g_object_notify (G_OBJECT (icon), "warning-color");
         }
 
-      color.red = prop.prop[9];
-      color.green = prop.prop[10];
-      color.blue = prop.prop[11];
+      color.red = prop.prop[9] / 65535.0;
+      color.green = prop.prop[10] /  65535.0;
+      color.blue = prop.prop[11] / 65535.0;
 
-      if (!gdk_color_equal (&icon->priv->success_color, &color))
+      if (!gdk_rgba_equal (&icon->priv->success_color, &color))
         {
           icon->priv->success_color = color;
 
@@ -633,6 +660,55 @@ gtk_tray_icon_get_padding_property (GtkTrayIcon *icon)
     XFree (prop.prop);
 }
 
+static void
+gtk_tray_icon_get_icon_size_property (GtkTrayIcon *icon)
+{
+  GdkScreen *screen = gtk_widget_get_screen (GTK_WIDGET (icon));
+  GdkDisplay *display = gdk_screen_get_display (screen);
+  Display *xdisplay = GDK_DISPLAY_XDISPLAY (display);
+
+  Atom type;
+  int format;
+  union {
+	gulong *prop;
+	guchar *prop_ch;
+  } prop = { NULL };
+  gulong nitems;
+  gulong bytes_after;
+  int error, result;
+
+  g_assert (icon->priv->manager_window != None);
+
+  gdk_error_trap_push ();
+  type = None;
+  result = XGetWindowProperty (xdisplay,
+			       icon->priv->manager_window,
+			       icon->priv->icon_size_atom,
+			       0, G_MAXLONG, FALSE,
+			       XA_CARDINAL,
+			       &type, &format, &nitems,
+			       &bytes_after, &(prop.prop_ch));
+  error = gdk_error_trap_pop ();
+
+  if (!error && result == Success &&
+      type == XA_CARDINAL && nitems == 1 && format == 32)
+    {
+      gint icon_size;
+
+      icon_size = prop.prop[0];
+
+      if (icon->priv->icon_size != icon_size)
+	{
+	  icon->priv->icon_size = icon_size;
+
+	  g_object_notify (G_OBJECT (icon), "icon-size");
+	}
+    }
+
+  if (type != None)
+    XFree (prop.prop);
+}
+
 static GdkFilterReturn
 gtk_tray_icon_manager_filter (GdkXEvent *xevent,
 			      GdkEvent  *event,
@@ -673,6 +749,11 @@ gtk_tray_icon_manager_filter (GdkXEvent *xevent,
         {
           gtk_tray_icon_get_padding_property (icon);
         }
+      else if (xev->xany.type == PropertyNotify &&
+               xev->xproperty.atom == icon->priv->icon_size_atom)
+        {
+          gtk_tray_icon_get_icon_size_property (icon);
+        }
       else if (xev->xany.type == DestroyNotify)
 	{
           GTK_NOTE (PLUGSOCKET,
@@ -782,6 +863,7 @@ gtk_tray_icon_update_manager_window (GtkTrayIcon *icon)
       gtk_tray_icon_get_visual_property (icon);
       gtk_tray_icon_get_colors_property (icon);
       gtk_tray_icon_get_padding_property (icon);
+      gtk_tray_icon_get_icon_size_property (icon);
 
       if (gtk_widget_get_realized (GTK_WIDGET (icon)))
 	{
@@ -1013,3 +1095,11 @@ _gtk_tray_icon_get_padding (GtkTrayIcon *icon)
 
   return icon->priv->padding;
 }
+
+gint
+_gtk_tray_icon_get_icon_size (GtkTrayIcon *icon)
+{
+  g_return_val_if_fail (GTK_IS_TRAY_ICON (icon), 0);
+
+  return icon->priv->icon_size;
+}
