@@ -60,21 +60,51 @@
 *tp++ = gdk_window_window_class_get_type();
 *tp++ = gdk_wm_decoration_get_type();
 *tp++ = gdk_wm_function_get_type();
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_app_launch_context_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_cursor_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_device_core_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_device_manager_core_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_device_manager_xi2_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_device_manager_xi_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_device_xi2_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_device_xi_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_display_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_display_manager_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_drag_context_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_keymap_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_screen_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_visual_get_type();
+#endif
+#ifdef GDK_WINDOWING_X11
 *tp++ = gdk_x11_window_get_type();
+#endif
 *tp++ = gtk_about_dialog_get_type();
 *tp++ = gtk_accel_flags_get_type();
 *tp++ = gtk_accel_group_get_type();
@@ -169,18 +199,12 @@
 *tp++ = gtk_file_filter_get_type();
 *tp++ = gtk_fixed_get_type();
 *tp++ = gtk_font_button_get_type();
-*tp++ = gtk_font_selection_dialog_get_type();
-*tp++ = gtk_font_selection_get_type();
+*tp++ = gtk_font_chooser_dialog_get_type();
+*tp++ = gtk_font_chooser_get_type();
 *tp++ = gtk_frame_get_type();
 *tp++ = gtk_gradient_get_type();
 *tp++ = gtk_grid_get_type();
 *tp++ = gtk_handle_box_get_type();
-*tp++ = gtk_hbox_get_type();
-*tp++ = gtk_hbutton_box_get_type();
-*tp++ = gtk_hpaned_get_type();
-*tp++ = gtk_hscale_get_type();
-*tp++ = gtk_hscrollbar_get_type();
-*tp++ = gtk_hseparator_get_type();
 *tp++ = gtk_hsv_get_type();
 *tp++ = gtk_icon_factory_get_type();
 *tp++ = gtk_icon_info_get_type();
@@ -209,6 +233,7 @@
 *tp++ = gtk_license_get_type();
 *tp++ = gtk_link_button_get_type();
 *tp++ = gtk_list_store_get_type();
+*tp++ = gtk_lock_button_get_type();
 *tp++ = gtk_menu_bar_get_type();
 *tp++ = gtk_menu_direction_type_get_type();
 *tp++ = gtk_menu_get_type();
@@ -227,6 +252,7 @@
 *tp++ = gtk_offscreen_window_get_type();
 *tp++ = gtk_orientable_get_type();
 *tp++ = gtk_orientation_get_type();
+*tp++ = gtk_overlay_get_type();
 *tp++ = gtk_pack_direction_get_type();
 *tp++ = gtk_pack_type_get_type();
 *tp++ = gtk_page_orientation_get_type();
@@ -236,7 +262,9 @@
 *tp++ = gtk_paper_size_get_type();
 *tp++ = gtk_path_priority_type_get_type();
 *tp++ = gtk_path_type_get_type();
+#ifdef GDK_WINDOWING_X11
 *tp++ = gtk_plug_get_type();
+#endif
 *tp++ = gtk_policy_type_get_type();
 *tp++ = gtk_position_type_get_type();
 *tp++ = gtk_print_context_get_type();
@@ -295,7 +323,9 @@
 *tp++ = gtk_size_group_get_type();
 *tp++ = gtk_size_group_mode_get_type();
 *tp++ = gtk_size_request_mode_get_type();
+#ifdef GDK_WINDOWING_X11
 *tp++ = gtk_socket_get_type();
+#endif
 *tp++ = gtk_sort_type_get_type();
 *tp++ = gtk_spin_button_get_type();
 *tp++ = gtk_spin_button_update_policy_get_type();
@@ -362,14 +392,8 @@
 *tp++ = gtk_ui_manager_get_type();
 *tp++ = gtk_ui_manager_item_type_get_type();
 *tp++ = gtk_unit_get_type();
-*tp++ = gtk_vbox_get_type();
-*tp++ = gtk_vbutton_box_get_type();
 *tp++ = gtk_viewport_get_type();
 *tp++ = gtk_volume_button_get_type();
-*tp++ = gtk_vpaned_get_type();
-*tp++ = gtk_vscale_get_type();
-*tp++ = gtk_vscrollbar_get_type();
-*tp++ = gtk_vseparator_get_type();
 *tp++ = gtk_widget_get_type();
 *tp++ = gtk_widget_help_type_get_type();
 *tp++ = gtk_widget_path_get_type();
